from kivy.factory import Factory

r = Factory.register
r("AKAddWidgetAnimationBehavior", module="akivymd.uix.behaviors.addwidget")
r("AKBadgeLayout", module="akivymd.uix.badgelayout")
r("AKFloatingRoundedAppbar", module="akivymd.uix.bottomappbar")
r("AKFloatingRoundedAppbarAvatarItem", module="akivymd.uix.bottomappbar")
r("AKFloatingRoundedAppbarButtonItem", module="akivymd.uix.bottomappbar")
r("AKBottomNavigation", module="akivymd.uix.bottomnavigation")
r("AKBottomNavigation2", module="akivymd.uix.bottomnavigation2")
r("Button_Item", module="akivymd.uix.bottomnavigation2")
r("AKImageLoader", module="akivymd.uix.imageloader")
r("AKLabelLoader", module="akivymd.uix.labelloader")
r("AKImageViewer", module="akivymd.uix.imageview")
r("AKImageViewerItem", module="akivymd.uix.imageview")
r("AKOnboarding", module="akivymd.uix.onboarding")
r("AKOnboardingItem", module="akivymd.uix.onboarding")
r("AKPieChart", module="akivymd.uix.piechart")
r("AKSelectList", module="akivymd.uix.selectionlist")
r("AKSelectListAvatarItem", module="akivymd.uix.selectionlist")
r("AKProgressbutton", module="akivymd.uix.progressbutton")
r("AKSilverAppbar", module="akivymd.uix.silverappbar")
r("AKSilverAppbarContent", module="akivymd.uix.silverappbar")
r("AKSilverAppbarHeader", module="akivymd.uix.silverappbar")
r("AKSpinnerDoubleBounce", module="akivymd.uix.spinners")
r("AKSpinnerCircleFlip", module="akivymd.uix.spinners")
r("AKSpinnerFoldingCube", module="akivymd.uix.spinners")
r("AKSpinnerThreeDots", module="akivymd.uix.spinners")
